/***************************************************************************
                          viswidget.cpp  -  description
                             -------------------
    begin                : Die Jan 7 2003
    copyright            : (C) 2003 by Mark Kretschmann
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "viswidget.h"

#include <math.h>
#include <vector>

#include <qevent.h>
#include <qframe.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpen.h>
#include <qpixmap.h>
#include <qwidget.h>

#include <kdebug.h>
#include <kstandarddirs.h>


VisWidget::VisWidget( QWidget *parent, const char *name ) :
   QFrame( parent, name ),
   m_pBgPixmap( 0 ),
   m_pSrcPixmap( 0 ),
   m_pComposePixmap( 0 ),
   m_pGridPixmap( 0 ),
   m_pRoofPixmap( 0 )
{}


VisWidget::~VisWidget()
{
    delete m_pBgPixmap;
    delete m_pSrcPixmap;
    delete m_pComposePixmap;
    delete m_pRoofPixmap;
    delete m_pGridPixmap;
}


// METHODS =====================================================

void VisWidget::polish()
{
    QWidget::polish();
    
    // we use polish for initialzing (instead of ctor), because we need to know the widget's final size
    init();
    initGrid();
}


void VisWidget::init()
{
    m_iVisHeight = height();
    m_pSrcPixmap = new QPixmap( m_iVisHeight*4, m_iVisHeight );
    m_pComposePixmap = new QPixmap( size() );

    m_pRoofPixmap = new QPixmap( 4, 1 );
    m_pRoofPixmap->fill( QColor( 0xff, 0x50, 0x70 ) );

    QPainter p( m_pSrcPixmap );
    p.setBackgroundColor( Qt::black );
    p.eraseRect( 0, 0, m_pSrcPixmap->width(), m_pSrcPixmap->height() );

    for ( uint x=0, r=0x40, g=0x30, b=0xff; x < m_iVisHeight; ++x )
    {
        for ( int y = x; y > 0; --y )
        {
            p.setPen( QPen( QColor( r + (+y*4), g+ (+y*0), b+ (-y*4) ) ) );
            p.drawLine( x * 4, m_iVisHeight - y, x * 4 + 4, m_iVisHeight - y );
        }
    }
}


void VisWidget::initGrid()
{
    m_pGridPixmap = new QPixmap( parentWidget()->paletteBackgroundPixmap()->convertToImage()
        .copy( this->x(), this->y(), width(), height() ) );

    QPainter painterGrid( m_pGridPixmap );
    painterGrid.setPen( QPen( QColor( 0x20, 0x20, 0x50 ) ) );

    for ( int x = 0; x < m_pGridPixmap->width(); x += 3 )
    {
        painterGrid.drawLine( x, 0, x, m_pGridPixmap->height()-1 );
    }

    for (int y = 0; y < m_pGridPixmap->height(); y += 3 )
    {
        painterGrid.drawLine( 0, y, m_pGridPixmap->width()-1, y );
    }
}


void VisWidget::initSin( std::vector<float> &v )
{
    double step = ( M_PI * 2 ) / SINVEC_SIZE;
    double radian = 0;

    for ( int i = 0; i < SINVEC_SIZE; i++ )
    {
        v.push_back( sin( radian ) );
        radian += step;
    }
}


// --------------------------------------------------------------------------------

#define BAND_COUNT 31

static int step = 5;

void VisWidget::drawAnalyzer( std::vector<float> *s )
{
    static std::vector<uint> barVector( BAND_COUNT, 0 );
    static std::vector<int>  roofVector( BAND_COUNT, 0 );
    static std::vector<uint> roofVelocityVector( BAND_COUNT, 0 );

    bitBlt( m_pComposePixmap, 0, 0, m_pGridPixmap ); //start with a blank canvas


    std::vector<float> bands( BAND_COUNT, 0 );
    std::vector<float>::const_iterator it( bands.begin() );

    if ( s ) interpolate( s, bands ); //if no s then we are paused/stopped

    for ( uint i = 0, x = 10, y2; i < bands.size(); ++i, ++it, x+=5 )
    {
        y2 = static_cast<uint>( ( *it ) * m_iVisHeight );

        if ( y2 > m_iVisHeight - 2 )
           y2 = m_iVisHeight - 2;

/*
        if ( (int)y2 > roofVector[i] )
        {
            roofVector[i] = (int)y2;
            roofVelocityVector[i] = 1;
        }
        else
        {
           //add some dynamics - makes the value closer to what it was last time
           //we do on ELSE as it looks better
           y2 = ( barVector[i] + y2 * 2 ) / 3;
        }
*/

        //add some dynamics - makes the value slightly closer to what it was last time
        if( step > 0 )
          y2 = ( barVector[i] + y2 * step ) / (step + 1);

        if ( (int)y2 > roofVector[i] )
        {
            roofVector[i] = (int)y2;
            roofVelocityVector[i] = 1;
        }


        //remember where we are
        barVector[i] = y2;

        //blt the coloured bar
        bitBlt( m_pComposePixmap, x, m_iVisHeight - y2,
                m_pSrcPixmap, y2 * 4, m_iVisHeight - y2, 4, y2, Qt::CopyROP );
        //blt the roof bar
        bitBlt( m_pComposePixmap, x, m_iVisHeight - roofVector[i] - 2, m_pRoofPixmap );


        //set roof parameters for the NEXT draw
        if ( roofVelocityVector[i] != 0 )
        {
            if ( roofVelocityVector[i] > 9 )
            {
               roofVector[i] -= roofVelocityVector[i] / 4;
            }

            if ( roofVector[i] < 0 )
            {
               roofVector[i] = 0; //not strictly necessary
               roofVelocityVector[i] = 0;
            }
            else ++roofVelocityVector[i];
        }
    }

    bitBlt( this, 0, 0, m_pComposePixmap );
}


void VisWidget::interpolate( std::vector<float> *oldVec, std::vector<float> &newVec )
{
    uint newSize = newVec.size(); //vector size is O(1)

    double pos = 0.0;
    double step = static_cast<double>( oldVec->size() ) / newSize;

    for ( uint i = 0; i < newSize; ++i, pos += step )
    {
        double error = pos - floor( pos );
        unsigned long offset = static_cast<unsigned long>( pos );

        unsigned long indexLeft = offset + 0;

        if ( indexLeft >= oldVec->size() )
            indexLeft = oldVec->size() - 1;

        unsigned long indexRight = offset + 1;

        if ( indexRight >= oldVec->size() )
            indexRight = oldVec->size() - 1;

        newVec[i] = oldVec->at( indexLeft ) * ( 1.0 - error ) +
                    oldVec->at( indexRight ) * error;
    }
}


void VisWidget::mouseReleaseEvent( QMouseEvent * )
{
    emit clicked();

/*    switch( e->button() ) {
    case LeftButton:
      step++;
      break;
    case MidButton:
      step--;
    default:
      break;
    }

    kdDebug() << "movingness factor: " <<  step << endl;*/
}


#include "viswidget.moc"
